#include <stdio.h>
#include "arg.h"
#include "log.h"
#include "dep.h"
#include "db.h"
#include "tmpasm_scconfig.h"
#include <string.h>

#define DEF_PREFIX    "/usr"
#define DEF_LEVELSDIR "$P/share/ncbricks/levels"
#define DEV_LEVELSDIR "levels"
#define DEF_GLOBALCFG "/etc/ncbricks"
#define DEV_GLOBALCFG "config.txt"
#define DEF_USERCFG   ".ncbricks_config"
/*#define DEF_USERSCORE ".ncbricks_scores"*/ /* TODO */

static void show_help();
static void enable_debug();

/* Runs when a custom command line argument is found
 returns true if no furhter argument processing should be done */
int hook_custom_arg(const char *key, const char *value)
{
	if(0==strcmp(key, "help"))
	{
		show_help();
		exit(0);
		return 0;
	}

	if(0==strcmp(key, "debug"))
	{
		enable_debug();
		return 1;
	}

	if(0==strcmp(key, "prefix"))
	{
		put("/local/ncbricks/prefix", value);
		return 1;
	}

	if(0==strcmp(key, "levelsdir"))
	{
		put("/local/ncbricks/levelsdir", value);
		return 1;
	}

	if(0==strcmp(key, "globalcfg"))
	{
		put("/local/ncbricks/globalcfg", value);
		return 1;
	}

	if(0==strcmp(key, "usercfg"))
	{
		put("/local/ncbricks/usercfg", value);
		return 1;
	}

	if(0==strcmp(key, "devel"))
	{
		enable_debug();
		put("/local/ncbricks/levelsdir", DEV_LEVELSDIR);
		put("/local/ncbricks/globalcfg", DEV_GLOBALCFG);
		put("/local/ncbricks/devel_mode", strue);
		return 1;
	}

	return 0;
}

static void show_help()
{
/*	FILE* const out = stdout;*/

	puts(
		"./configure [ARG]...\n"
		"Configure ncbricks\n"
		"\n"
		"  --prefix=PATH\n"
		"      set prefix to PATH\n"
		"      default: " DEF_PREFIX "\n"
		"  --levelsdir=PATH\n"
		"      set levels/ directory to PATH (\"$P\" will be replaced with prefix)\n"
		"      default: " DEF_LEVELSDIR "\n"
		"  --globalcfg=PATH\n"
		"      set global configuration file to PATH (\"$P\" will be replaced with prefix)\n"
		"      default: " DEF_GLOBALCFG "\n"
		"  --usercfg=PATH\n"
		"      set per-user configuration file to PATH\n"
		"      default: " DEF_USERCFG "\n"
		"  --devel\n"
		"      enable developer options (implies --debug)\n"
		"      sets levelsdir to " DEV_LEVELSDIR "\n"
		"      sets globalcfg to " DEV_GLOBALCFG "\n"
		"  --debug\n"
		"      enable debug mode (more warnings, be pedantic as possible)\n"
	);
}

static void enable_debug()
{
	put("/local/ncbricks/debug", strue);
}

/* Runs before anything else */
int hook_preinit()
{
	return 0;
}

/* Runs after initialization */
int hook_postinit()
{
	db_mkdir("/local");
	put("/local/ncbricks/debug", sfalse);
	put("/local/ncbricks/prefix", DEF_PREFIX);
	put("/local/ncbricks/levelsdir", DEF_LEVELSDIR);
	put("/local/ncbricks/globalcfg", DEF_GLOBALCFG);
	put("/local/ncbricks/usercfg", DEF_USERCFG);
	return 0;
}

/* Runs after all arguments are read and parsed */
int hook_postarg()
{
	return 0;
}

/* Runs when things should be detected for the host system */
int hook_detect_host()
{
	return 0;
}

/* Runs when things should be detected for the target system */
int hook_detect_target()
{
	if(istrue(get("/local/ncbricks/debug")))
	{
		require("cc/argstd/Wall", 0, 0);
		require("cc/argstd/Wextra", 0, 0);
		/*require("cc/argstd/std_c89", 0, 0);*/
	}

	require("cc/cc", 0, 0);
	require("libs/tty/ncurses/cflags", 0, 0);
	require("libs/tty/ncurses/includes", 0, 0);
	require("libs/tty/ncurses/ldflags", 0, 0);
	require("sys/path_sep", 0, 0);
	return 0;
}

/* Runs after detection hooks, should generate the output (Makefiles, etc.) */
int hook_generate()
{
	const char* const root = "../src";
	int generr = 0;

	fprintf(stderr, "Generating ncbricks/Makefile (%d)\n", generr |= tmpasm(root, "Makefile.in", "Makefile"));
	fprintf(stderr, "Generating ncbricks/config.h (%d)\n", generr |= tmpasm(root, "config.h.in", "config.h"));

	return generr;
}

/* Runs before everything is uninitialized */
void hook_preuninit()
{
}

/* Runs at the very end, when everything is already uninitialized */
void hook_postuninit()
{
}

